#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "aa.h"
#include "doublevector.h"
#include "blosum.h"


/************************************************************************
*  Define AA Frequencies
*    We want cond_{ij} to be the probability of seeing $i$ given that you
*  have seen $j$.  Thus, columns should add up.
************************************************************************/

void
Blosum_read (double **joint, double **cond, char *filename) {
  FILE *fp;
  double fij, total, *sum;
  int i, j, stdi, stdj, len;
  char Buffer[200];
  
  fp = fopen(filename,"r");
  if (!fp) {
    printf("Cannot open Blosum file %s.\n",filename);
    exit(1);
  }
  
  sum = Doublevector_new(NAT_AAS);
  for (i = 0; i < 10; ++i) {	/* Read 10 lines of comments */
    fgets(Buffer,200,fp);
  }

  for (i = 0; i < NAT_AAS; ++i) {
    stdi = AA_acd_index(AA_arn_aa(i));
    for (j = 0; j <= i; ++j) {
      stdj = AA_acd_index(AA_arn_aa(j));
      fscanf(fp,"%lf",&fij);
      joint[stdi][stdj] += fij/2.0;
      joint[stdj][stdi] += fij/2.0;
    }
  }
  fclose(fp);

  for (j = 0; j < NAT_AAS; ++j) {
    sum[j] = 0.0;
    for (i = 0; i < NAT_AAS; ++i) {
      sum[j] += joint[i][j];
    }
    for (i = 0; i < NAT_AAS; ++i) {
      cond[i][j] = joint[i][j]/sum[j];
    }
    total += sum[j];
  }
  Doublevector_free(sum);

  for (j = 0; j < NAT_AAS; ++j) {
    for (i = 0; i < NAT_AAS; ++i) {
      joint[i][j] /= total;
    }
  }
}


