#include <stddef.h>
#include <stdlib.h>
#include "intlist.h"
#include "tally.h"
#include "weighting.h"

Vector_T
Weighting_uniform_new (Block_T block) {
  int seq;
  double wt = 1.0;
  Vector_T weights;

  weights = Vector_new(Block_sequences(block));
  for (seq = 0; seq < Block_sequences(block); ++seq) {
    Vector_put(weights,seq,wt);
  }
  return weights;
}


Vector_T
Weighting_henikoff_new (Block_T block) {
  Tally_T tally;
  Intlist_T s;
  int pos, n_positions, n_residues, n_seqs_in_res, total_seqs;
  double wt;
  Vector_T weights;

  weights = Vector_new(Block_sequences(block));
  total_seqs = Block_sequences(block);
  n_positions = Block_positions(block);
  for (pos = 0; pos < n_positions; ++pos) {
    tally = Block_get(block,pos);
    n_residues = Tally_length(tally);
    for ( ; tally; tally = Tally_rest(tally)) {
      n_seqs_in_res = Tally_count(tally);
      for (s = Tally_sequences(tally); s; s = s->rest) {
	wt = Vector_get(weights,s->first);
	wt += (double) total_seqs/
	  (double) (n_residues*n_seqs_in_res*n_positions);
	Vector_put(weights,s->first,wt);
      }
    }
  }
  return weights;
}


