#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "debug.h"
#include "aa.h"
#include "cdf.h"

#if DEBUG
static double
check_pdf (double *ProbX, int length) {
  int i;
  double prob = 0.0;

  for (i = 0; i < length; i++) {
    prob += ProbX[i];
  }
  return prob;
}
#endif


static int
min_column (int **profile, int position, int numres) {
  int res, min;

  min = profile[position][0];
  for (res = 1; res < numres; res++) {
    if (profile[position][res] < min) {
      min = profile[position][res];
    }
  }
  return min;
}

static int
max_column (int **profile, int position, int numres) {
  int res, max;
  
  max = profile[position][0];
  for (res = 1; res < numres; res++) {
    if (profile[position][res] > max) {
      max = profile[position][res];
    }
  }
  return max;
}


void
CDF_bounds (int *min, int *max, int *min_scores, int *max_scores, 
	    int **profile, int width) {
  int i;

  *min = min_scores[0] = min_column(profile,0,NAT_AAS);
  *max = max_scores[0] = max_column(profile,0,NAT_AAS);
  for (i = 1; i < width; i++) {
    min_scores[i] = min_scores[i-1] + min_column(profile,i,NAT_AAS);
    max_scores[i] = max_scores[i-1] + max_column(profile,i,NAT_AAS);
  }
  for (i = 1; i < width; i++) {
    if (min_scores[i] < *min) {
      *min = min_scores[i];
    }
    if (max_scores[i] > *max) {
      *max = max_scores[i];
    }
  }
}


static void
reset_pdf (double *pdfX, int length) {
  int i;
  
  for (i = 0; i < length; i++) {
    pdfX[i] = 0.0;
  }
}

static void
compute_probabilities (double *pdf, int **profile, int width,
		       int min_score, int max_score) {
  int length, i, j, aa, score;
  double *pdfEven, *pdfOdd;

  length = max_score - min_score + 1;
  if ((width-1)%2 == 0) {
    pdfEven = pdf;
    pdfOdd = (double *) calloc(length,sizeof(double));
  } else {
    pdfOdd = pdf;
    pdfEven = (double *) calloc(length,sizeof(double));
  }

  pdfOdd[0-min_score] = 1.0;
  for (j = 0; j < width; j++) {
    if (j%2 == 1) {
      reset_pdf(pdfOdd,length);
      for (i = 0; i < length; i++) {
	if (pdfEven[i] > 0.0) {
	  for (aa = 0; aa < NAT_AAS; aa++) {
	    score = profile[j][aa];
	    pdfOdd[i+score] = pdfOdd[i+score] + pdfEven[i]*AA_acd_freq(aa);
	  }
	}
      }
      debug(printf("Iteration %d pdfOdd total = %f\n",j,check_pdf(pdfOdd,length)));
    } else {
      reset_pdf(pdfEven,length);
      for (i = 0; i < length; i++) {
	if (pdfOdd[i] > 0.0) {
	  for (aa = 0; aa < NAT_AAS; aa++) {
	    score = profile[j][aa];
	    pdfEven[i+score] = pdfEven[i+score] + pdfOdd[i]*AA_acd_freq(aa);
	  }
	}
      }
      debug(printf("Iteration %d pdfEven total = %f\n",j,check_pdf(pdfEven,length)));
    }
  }

  if ((width-1)%2 == 0) {
    free(pdfOdd);
  } else {
    free(pdfEven);
  }
}


void
CDF_compute (double *G, int **calib_matrix, int width,
	     int min_score, int max_score) {
  int length, i;
  double prob, sum = 0.0;
  double *pdf;

  length = max_score - min_score + 1;
  pdf = (double *) calloc(length,sizeof(double));
  compute_probabilities(pdf,calib_matrix,width,min_score,max_score);
  for (i = length-1; i >= 0; --i) {
    if ((prob = pdf[i]) > 0.0) {
      G[i] = sum += prob;
    } else {
      G[i] = -1.0;
    }
  }
  free(pdf);
}


int
CDF_getThreshold (double *G, double p_threshold, int min_score, int max_score) {
  int length, i;

  length = max_score - min_score + 1;
  for (i = length-1; i >= 0; --i) {
    if ((G[i] != -1.0) && (p_threshold < G[i])) {
      return min_score+i;
    }
  }
  return min_score;
}

