/*
 * File:  sequence.h
 * Date:  09-07-99
 * Programmers:  Thomas Wu (thomas.wu@Stanford.Edu)
 *               Craig Nevill-Manning (nevill@cs.rutgers.edu)
 * Comment Writer:  Jimmy Y. Huang (yhuang@leland.stanford.edu)
 * ------------------------------------------------------------
 * This file exports the Sequence_read function that facilitates the reading
 * and processing of input sequences for emotif-search.
 * ------------------------------------------------------------
 * Copyright 1999 The Board of Trustees of 
 * The Leland Stanford Junior University. All Rights Reserved.
 */


#ifndef SEQUENCE_INCLUDED
#define SEQUENCE_INCLUDED
#include <stdio.h>


/*
 * ----------
 * Sequence_read
 * ==========
 * This function takes three arguments.  The first argument is a pointer to
 * the file containing the input sequences.  The second argument is a
 * pointer to the user-supplied char pointer name; this char **name, after
 * the execution of the Sequence_read function, will point to a dynamic
 * array containing the name of one input sequence.  The third argument is
 * a pointer to the user-supplied int variable length; this int variable
 * length, after the execution of the Sequence_read function, will
 * contain the length of the input sequence just read.  The function
 * Sequence_read returns a char pointer to a dynamic array containing one
 * input sequence from the user-supplied file.  Clients of the Sequence_read
 * function should realize that this function allocates memory on the heap
 * "behind-the-scene," but the memory de-allocation responsibilities lie
 * with the users of this library.  The function Sequence_read returns NULL
 * if no sequence has been read.
 * ----------
 */

extern char *Sequence_read (FILE *fp, char **name, int *length);

#endif



